//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package exgwe.sql;

import exjava.sql.*;
import exgwe.sql.gweMysql;
import exgwe.sql.gweMysqlColumn;
import exgwe.sql.gweMysqlException;
import exjava.math.BigDecimal;
import exjava.math.BigInteger;


/**
 * This class implements the JDBC draft specification for the ResulSet
 * interface for mysql.  The class is designed to hold the results of
 * database queries.
 */
public class gweMysqlResultSet implements exjava.sql.ResultSet {
  private gweMysql result;
  /**
   * The meta data for this result set.
   */
  private gweMysqlMetaResult meta;
  /**
   * Keeps track of the last column read. 
   */
  private int last_read = -1;
  
  /**
   * Constructs a new result set object given the gweMysqlResult specified.
   * @param res the gweMysql returned from a previously executed query
   */
  public gweMysqlResultSet(gweMysql res) {
    result = res;
  }

  /**
   * JDBC draft specification method for moving the current row to the
   * next row, returning true if there is another row for processing.
   * @see exjava.sql.ResultSet#next
   * @exception SQLException thrown if an error occurs during processing
   * @return true if there are more rows to process, otherwise false
   */
  public synchronized boolean next() throws SQLException {
    try {
		last_read=-1;
		return result.nextResult();
    } catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }
    
  /**
   * JDBC specification method to determine if a column is null.
   * @see exjava.sql.ResultSet#isNull
   * @exception SQLException in the event of an gweMysqlException
   * @param column the column being tested for nullness
   * @return true if the column is null, false otherwise
   */
  public boolean isNull(int column) throws SQLException {
    try {
		return (result.isNull(column));
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC specification method to determine if the last column read was null.
   * @see exjava.sql.ResultSet#wasNull
   * @exception SQLException in the event of an gweMysqlException
   * @return true if the column was null, false otherwise
   */
  public boolean wasNull() throws SQLException {
    if( last_read == -1 ) return false;
    return isNull(last_read);
  }

  /**
   * JDBC draft specification method for getting a string value from
   * the named column.  Note that the JDBC draft provides that this
   * method gets the value as a char, so you can retrieve int values
   * into String objects.
   * @see exjava.sql.ResultSet#getString
   * @exception SQLException thrown for invalid columns or bad rows
   * @param column the column being retrieved
   * @return the column as a String
   */
  public String getString(int column) throws SQLException {
    last_read = column;
    if( isNull(column) ) return null;
    try {
		return result.getString(column);
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * Get the value of a column in the current row as a Java byte.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL the result is 0
   */
  public byte getByte(int column) throws SQLException {
    String str;
    
    last_read = column;
    if( isNull(column) ) return 0;
    try {
	    str = result.getString(column);
	    if( str.equals("") ) return 0;
	    return (byte)(Integer.valueOf(str).intValue());
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }
  
  /**
   * JDBC specification method for retrieving a column as a boolean
   * value.  Interprets "", null, and "0" as false, others as true.
   * @see exjava.sql.ResultSet#getBoolean
   * @exception SQLException a sure sign of the apocolypse
   * @param column the column for which the value is being retrieved
   * @return false for "", null, or "0"; true otherwise
   */
  public boolean getBoolean(int column) throws SQLException {
    last_read = column;
    if( isNull(column) ) return false;
    try {
      String str = result.getString(column);
      
      if( str.equals("") ) return false;
      if( str.equals("0") ) return false;
      if( str.equals("N") ) return false;
      if( str.equals("n") ) return false;
      return true;
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method to retrieve a short value from
   * the database.
   * @see exjava.sql.ResultSet#getShort
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a short
   */
  public short getShort(int column) throws SQLException {
    try {
		String str;
		last_read=column;
		str = result.getString(column);
		if( str == null ) return 0;
	    return (short)(Integer.valueOf(str).intValue());
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }  

  /**
   * JDBC draft specification method to retrieve an integer value from
   * the database.
   * @see exjava.sql.ResultSet#getInt
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as an integer
   */
  public int getInt(int column) throws SQLException {
    try {
		String str;
		last_read=column;
		str = result.getString(column);
		if( str == null ) return 0;
  	    return Integer.valueOf(str).intValue();
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }  
  
  /**
   * JDBC draft specification method to retrieve a long value from
   * the database.
   * @see exjava.sql.ResultSet#getLong
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a short
   */
  public long getLong(int column) throws SQLException {
    try {
		String str;
		last_read=column;
		str = result.getString(column);
		if( str == null ) return 0;
	    return Long.valueOf(str).longValue();
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }  

  /**
   * JDBC draft specification method to retrieve a float value from
   * the database.
   * @see exjava.sql.ResultSet#getFloat
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a float
   */
  public float getFloat(int column) throws SQLException {
    try {
		String str;
		last_read=column;
		str = result.getString(column);
		if( str == null ) return 0;
      	return Float.valueOf(str).floatValue();
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }  

  /**
   * JDBC draft specification method to retrieve a double value from
   * the database.
   * @see exjava.sql.ResultSet#getDouble
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a double
   */
  public double getDouble(int column) throws SQLException {
    try {
	    String str;
		last_read=column;
    	str = result.getString(column);
    	if( str == null ) return 0;
      	return Double.valueOf(str).doubleValue();
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }  


  public BigDecimal getBigDecimal(int column, int scale) throws SQLException {
    try {
		String str;
		last_read=column;
		str = result.getString(column);
		if( str == null ) return new BigDecimal (new BigInteger ("0"), scale);
		else return new BigDecimal (new BigInteger (str), scale);
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }


  public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException {
    try {
		String str;
		str = result.getString(columnName);
		if( str == null ) return new BigDecimal (new BigInteger ("0"), scale);
		else return new BigDecimal (new BigInteger (str), scale);
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }


  /**
   * JDBC draft specification method to retrieve a Bignum object from
   * the database.
   * @see exjava.sql.ResultSet#getBignum
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @param scale how many decimal digits after the floating point to maintain
   * @return the named column as a Bignum
  public Bignum getBignum(int column, int scale) throws SQLException {
    try {
		String str;
		str = result.getString(column);
		if( str == null ) return new Bignum(0, scale);
		else return new Bignum(str, scale);
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }
   */


  /**
   * JDBC draft specification method to retrieve a decimal from
   * the database.
   * @see exjava.sql.ResultSet#getDecimal
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @param scale how many decimal digits after the floating point to maintain
   * @return the named column as a Bignum
  public Bignum getDecimal(int column, int scale) throws SQLException {
    try {
		String str;
		str = result.getString(column);
		if( str == null ) return new Bignum(0, scale);
		else return new Bignum(str, scale);
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }
   */


  /**
   * JDBC draft specification method to return a byte array.
   * @see exjava.sql.ResultSet#getBytes
   * @exception SQLException thrown if something goes wrong
   * @param column the column being retrieved
   * @return a byte array that is the value of the column
   */

  public byte[] getBytes(int column) throws SQLException {
	try {
		return result.getBytes(column);
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }

  }

  /**
   * JDBC 1.10 specification for retrieving a date column.
   * Can you say namespace pollution?  I knew you could.
   * @see exjava.sqlResultSet#getDate
   * @exception SQLException thrown in the event of problems
   * @param column the column being retrieved
   * @return the date value for the column
   */
  public exjava.sql.Date getDate(int column)
       throws SQLException {
	try {
		String str;
		str = result.getString(column);
		if( str == null ) return null;
		Integer y = new Integer(str.substring(0,2));
		Integer m = new Integer(str.substring(2,4));
		Integer d = new Integer(str.substring(4,6));
		return new exjava.sql.Date(y.intValue(), m.intValue()-1, d.intValue());
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method for retrieving a time from the database.
   * @see exjava.sql.ResultSet#getTime
   * @exception SQLException thrown in the event of troubles
   * @param column the column being retrieved
   * @return the column as a exjava.sql.Time object
   */
  public exjava.sql.Time getTime(int column)
       throws SQLException {
	try {
		String str;
		str = result.getString(column);
		if( str == null ) return null;
		Integer h = new Integer(str.substring(6,8));
		Integer m = new Integer(str.substring(8,10));
		Integer s = new Integer(str.substring(10,12));
		return new exjava.sql.Time(h.intValue(), m.intValue(), s.intValue());
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method for retrieving a timestamp from
   * the database.
   * @see exjava.sql.ResultSet#getTimestamp
   * @exception SQLException thrown in the event of troubles
   * @param column the column being retrieved
   * @return the column as a exjava.sql.Timestamp object
   */
  public exjava.sql.Timestamp getTimestamp(int column)
       throws SQLException {
	try {
		String str;
		str = result.getString(column);
		if( str == null ) return null;
		Integer y = new Integer(str.substring(0,2));
		Integer m = new Integer(str.substring(2,4));
		Integer d = new Integer(str.substring(4,6));
		Integer hr = new Integer(str.substring(6,8));
		Integer mi = new Integer(str.substring(8,10));
		Integer se = new Integer(str.substring(10,12));
		return new exjava.sql.Timestamp(y.intValue(), m.intValue()-1, d.intValue(), hr.intValue(), mi.intValue(), se.intValue(), 0);
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * This is not currently supported.
   */
  public java.io.InputStream getAsciiStream(int column)
       throws SQLException {
    return null;
  }

  /**
   * This is not currently supported.
   */
  public java.io.InputStream getUnicodeStream(int column)
       throws SQLException {
    return null;
  }

  /**
   * This is not currently supported.
   */
  public java.io.InputStream getBinaryStream(int column)
       throws SQLException {
    return null;
  }
  
  /**
   * JDBC draft specification method for closing a result set.
   * @see exjava.sql.ResultSet#close
   */
  public void close() throws SQLException {
	// We need to halt the flow of result packets - 
	// for now, close the socket, and re-conect.
	try {

		result.reConnect();

	}
    catch( Exception e ) {
		throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method for returning a cursor name.
   * @see exjava.sql.ResultSet#getCursorName
   * @return ""
   */
  public String getCursorName() throws SQLException {
    return "";
  }

  /**
   * JDBC draft specification method for returning meta-deta on a result
   * set.
   * @see exjava.sql.ResultSet#getMetaData
   * @exception SQLException thrown on error getting meta-data
   * @return ResultSetMetaData object containing result set info
   */
  public ResultSetMetaData getMetaData()
       throws SQLException {
    if( meta == null ) {
      meta = new gweMysqlMetaResult(result);
    }
    return meta;
  }

  /**
   * JDBC draft specification method for retrieving data as objects.
   * @see exjava.sql.ResultSet#getObject
   * @exception SQLException in the event of retrieval! error
   * @param column the column desired
   * @param type the SQL data type of the field
   * @scale preceision for Bignums
   * @return the column specified as an Object
   */
  public Object getObject(int column, int type, int scale)
       throws SQLException {
    switch(type) {
    case Types.BIT:
      return new Boolean(getBoolean(column));

    case Types.TINYINT:
      return new Character((char)getInt(column));

    case Types.SMALLINT:
      return new Integer(getShort(column));

    case Types.INTEGER:
      return new Integer(getInt(column));

    case Types.BIGINT:
      return new Long(getLong(column));

    case Types.FLOAT:
      return new Float(getFloat(column));

    case Types.REAL:
      return new Float(getFloat(column));

    case Types.DOUBLE:
      return new Double(getDouble(column));

    case Types.NUMERIC:
      return getBigDecimal(column, scale);

    case Types.DECIMAL:
      return getBigDecimal(column, scale);

    case Types.CHAR:
      return getString(column);

    case Types.VARCHAR:
      return getString(column);

    case Types.LONGVARCHAR:
      return getString(column);

    case Types.DATE:
      return getDate(column);

    case Types.TIME:
      return getTime(column);

    case Types.TIMESTAMP:
      return getTimestamp(column);

    case Types.BINARY:
      return getBytes(column);

    case Types.VARBINARY:
      return getBytes(column);

    case Types.LONGVARBINARY:
      return getBytes(column);

    default:
      return null;
    }
  }

  /**
   * Same as above, except with a default scale to 0.
   */
  public Object getObject(int column, int type)
       throws SQLException {
    return getObject(column, type, 0);
  }

  /**
   * Same as above, except using the column's default SQL type.
   */
  public Object getObject(int column) throws SQLException {
    ResultSetMetaData meta = getMetaData();
    int type = meta.getColumnType(column);

    return getObject(column, type);
  }
    
  /**
   * JDBC 0.65 specification method for retrieving a column value
   * as a char based on the column name.
   * @see exjava.sql.ResultSet#getString
   * @param name the name of the column desired
   * @return the value of the column as a char
   */
  public String getString(String name) throws SQLException {
    return getString(findColumn(name));
  }

  /**
   * Returns the column as a byte based on column name
   */
  public byte getByte(String columnName) throws SQLException {
    return getByte(findColumn(columnName));
  }
  
  /**
   * Get the value of a BIT column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is false
   */
  public boolean getBoolean(String columnName) throws SQLException {
    return getBoolean(findColumn(columnName));
  }
  
  /**
   * Get the value of a short by column name
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public short getShort(String columnName) throws SQLException {
    return getShort(findColumn(columnName));
  }

  /**
   * Get the value of a INTEGER column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public int getInt(String columnName) throws SQLException {
    return getInt(findColumn(columnName));
  }
  
  /**
   * Get the value of a long column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public long getLong(String columnName) throws SQLException {
    return getLong(findColumn(columnName));
  }

  /**
   * Get the value of a FLOAT column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public float getFloat(String columnName) throws SQLException {
    return getFloat(findColumn(columnName));
  }

  /**
   * Get the value of a DOUBLE column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public double getDouble(String columnName) throws SQLException {
    return getDouble(findColumn(columnName));
  }
  

  /**
   * Get the value of a NUMERIC column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null

  public Bignum getBignum(String columnName, int scale) throws SQLException {
    return getBignum(findColumn(columnName), scale);
  }
   */


  /**
   * Get the value of a BINARY column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public byte[] getBytes(String columnName) throws SQLException {
    return getBytes(findColumn(columnName));
  }
  
  /**
   * Get the value of a DATE column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public exjava.sql.Date getDate(String columnName) throws SQLException {
    return getDate(findColumn(columnName));
  }
  
  /**
   * Get the value of a TIME column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public exjava.sql.Time getTime(String columnName) throws SQLException {
    return getTime(findColumn(columnName));
  }
  
  /**
   * Get the value of a TIMESTAMP column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public exjava.sql.Timestamp getTimestamp(String columnName)
       throws SQLException {
    return getTimestamp(findColumn(columnName));
  }
  
  /**
   * Very large ascii values in a LONGVARCHAR column can
   * be read in small chunks from a java.io.InputStream.
   *
   * <P><B>Note:</B> All the data in the returned stream must
   * be read prior to getting the value of any other column. The
   * next call to a get method implicitly closes the stream.
   *
   * @param columnName is the SQL name of the column
   * @return the java input stream which contains the ascii value 
   */
  public java.io.InputStream getAsciiStream(String columnName)
       throws SQLException {
    return getAsciiStream(findColumn(columnName));
  }

  /**
   * Very large unicode values in a LONGVARCHAR column can
   * be read in small chunks from a java.io.InputStream.
   *
   * <P><B>Note:</B> All the data in the returned stream must
   * be read prior to getting the value of any other column. The
   * next call to a get method implicitly closes the stream.
   *
   * @param columnName is the SQL name of the column
   * @return the java input stream which contains the unicode value 
   */
  public java.io.InputStream getUnicodeStream(String columnName)
       throws SQLException {
    return getUnicodeStream(findColumn(columnName));
  }

  /**
   * Very large binary values in a LONGVARBINARY column can
   * be read in small chunks from a java.io.InputStream.
   *
   * <P><B>Note:</B> All the data in the returned stream must
   * be read prior to getting the value of any other column. The
   * next call to a get method implicitly closes the stream.
   *
   * @param columnName is the SQL name of the column
   * @return the java input stream which contains the binary value */
  public java.io.InputStream getBinaryStream(String columnName)
       throws SQLException {
    return getBinaryStream(findColumn(columnName));
  }

  /**
   * Get the value of a column as an object; an integral value is
   * returned as its java.lang equivalent object; a LONGVARCHAR or
   * LONGVARBINARY value is returned as a java.io.InputStream.
   *
   * @param columnName the SQL column name
   * @param sqlType SQL type code defined by exjava.sql.Types
   * @return the parameter value; if isNull the value is null 
   */
  public Object getObject(String columnName, int sqlType, int scale)
       throws SQLException {
    return getObject(findColumn(columnName), sqlType, scale);
  }

  /**
   * Same as above, except defaulting scale to 0.
   */
  public Object getObject(String columnName, int type)
       throws SQLException {
    return getObject(findColumn(columnName), type, 0);
  }

  /**
   * Same as above, except returning the default SQL type
   */
  public Object getObject(String columnName) throws SQLException {
    return getObject(findColumn(columnName));
  }
  
  /**
   * column name counter part for other isNull()
   */
  public boolean isNull(String name) throws SQLException {
    return isNull(findColumn(name));
  }

  /**
   * Given a column name, this method returns the column number for that
   * name.  Column name to number mappings are kept inside a Hashtable.
   * Applications that do not need the overhead of this calculation are
   * not penalized since the mapping only occurs on the first attempt to
   * access a column number by name.
   * @exception exjava.sql.SQLException thrown if a bad name is passed
   * @param name the name of the column desired
   * @return the column number, 1 being the first column
   */
  public int findColumn(String name) throws SQLException {
	try {
		return result.getColumnNum(name);
	}
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }
  /**
   * JDBC draft specification for getting the chain of warnings for this
   * statement.
   * @see exjava.sql.Statement#getWarnings
   * @return the chain of warnings
   */
  public SQLWarning getWarnings() throws SQLException {
    return null;
  }

  /**
   * JDBC draft specification for clearing the warning chain.
   * @see exjava.sql.Statement#clearWarnings
   */
  public void clearWarnings() throws SQLException {
  }
}
