//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 12.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package exgwe.sql;

import exjava.sql.*;
import exgwe.sql.gweMysql;
import exgwe.sql.gweMysqlException;

/**
 * This class is the implementation of the JDBC draft specification
 * Statement interface.  Statement objects are used for executing SQL
 * statements.
 */
public class gweMysqlStatement implements exjava.sql.Statement {
  /**
   * A connection object to direct information to.
   */
    private gweMysqlConnection connection;
  /**
   * A result set returned from this query or whatever.
   */
  private gweMysqlResultSet result;
  /**
   * The max field size for mysql
   * NOTE: This is the value of MAXBUFFER in gweMysql.packet
   */
  private int max_field_size = 65535;
  /**
   * The max rows supported by mysql - big!
   */
  private int max_rows = 50000000; 	//Because it's in the FAQ!
  /**
   * The number of seconds the driver will allow for a SQL statement to
   * execute before giving up.  The default is to wait forever (0).
   */
  private int timeout = 0;
  
  /**
   * Constructs a new gweMysqlStatement object.
   * @param conn the gweMysqlConnection object
   */
  public gweMysqlStatement(gweMysqlConnection conn) {
    connection = conn;
  }
  
  /**
   * JDBC draft specification method to execute the specified SQL
   * query and gives back a result set.
   * @see exjava.sql.Statement#executeQuery
   * @exception SQLException raised for any gweMysqlException
   * @param sql the SQL statement string
   * @return the result set from the query
   */
  public ResultSet executeQuery(String sql)
       throws SQLException {
    result = null;
    return new gweMysqlResultSet(connection.executeQuery(sql));
  }

  /**
   * JDBC 0.54 specification method for executing simple UPDATE, INSERT,
   * DELETE, etc. statements which do not return ResultSet's.
   * @see exjava.sql.Statement#executeUpdate
   * @exception exjava.sql.SQLException thrown when an error occurs executing
   * the SQL
   * @return 0
   */
  public int executeUpdate(String sql) throws SQLException {
    return connection.executeUpdate(sql);
  }
  
  /**
   * JDBC draft specification method to execute an SQL statement and return
   * true if a result set was gotten, false otherwise.  
   * @see exjava.sql.Statement#execute
   * @exception SQLException raised for any gweMysqlException encountered
   * @param sql the SQL to be executed
   * @return true if there is a result set available
   */
  public boolean execute(String sql) throws SQLException {
    gweMysql r;
    
	r = connection.executeQuery(sql);
    if( r == null ) {
      result = null;
    }
    else {
      result = new gweMysqlResultSet(r);
    }
    return (result != null);
  }

  /**
   * JDBC draft specification method to close off any open result sets
   * for this Statement.  
   * @see exjava.sql.Statement#close
   */
  public void close() throws SQLException {
  }
  
  /**
   * JDBC 0.54 specification method for obtaining a result set from a
   * statement object.
   * @see exjava.sql.Statement#getResultSet
   * @return null if no result set is available, otherwise a result set
   */
  public ResultSet getResultSet() throws SQLException {
    ResultSet r;

    r = result;    
    result = null; 
    return r;
  }

  /**
   * Returns -1 always for mysql.
   * @see exjava.sql.Statement#getUpdateCount
   * @return -1
   */
  public int getUpdateCount() throws SQLException {
    return -1;
  }

  /**
   * JDBC 0.54 specification for determining if any more result sets
   * are left from a database statement. 
   * @see exjava.sql.Statement#getMoreResults
   * @return true if rows are to be gotten
   */
  public boolean getMoreResults() throws SQLException {
    return (result != null);
  }

  /**
   * JDBC draft specification method for determining the number of
   * results sets left to process for multiple result set processing.
   * @see exjava.sql.Statement#getMoreCounts
   * @see getMoreResults
   * @return -1
   */
  public int getMoreCounts() throws SQLException {
    return -1;
  }

  /**
   * JDBC draft specification method for getting the max field size
   * @see exjava.sql.Statement#getMaxFieldSize
   * @return the value of max field size
   */
  public int getMaxFieldSize() throws SQLException {
    return max_field_size;
  }

  /**
   * JDBC draft specification method for setting the max field size.
   * @see exjava.sql.Statement#setMaxFieldSize
   * @param max the maximum field size
   */
  public void setMaxFieldSize(int max) throws SQLException {
    max_field_size = max;
  }

  /**
   * JDBC draft specification method for getting the max rows supported
   * @see exjava.sql.Statement#getMaxRows
   * @return the maximum rows supported by mysql
   */
  public int getMaxRows() throws SQLException {
    return max_rows;
  }

  /**
   * JDBC draft specification method for setting the max rows.
   * @see exjava.sql.Statement.setMaxRows
   * @param max the max rows
   */
  public void setMaxRows(int max) throws SQLException {
    max_rows = max;
  }

  /**
   * JDBC draft specification method for escape processing.
   * @see exjava.sql.Statement#setEscapeProcessing
   * @param enable this does nothing right now
   */
  public void setEscapeProcessing(boolean enable)
       throws SQLException {
    throw new SQLException("mysql does not support escape processing.");
  }

  /**
   * JDBC draft specification method for getting the query timeout, which
   * is the number of seconds the driver waits before giving up on the
   * query.
   * @see exjava.sql.Statement#getQueryTimeout
   * @see setQueryTimeout
   * @return the timeout value for this statement
   */
  public int getQueryTimeout() throws SQLException {
    return timeout;
  }

  /**
   * JDBC draft specification method for setting the query timeout.
   * @see exjava.sql.Statement#setQueryTimeout
   * @see getQueryTimeout
   * @param x the new query timeout value
   */
  public void setQueryTimeout(int x) throws SQLException {
    timeout = x;
  }

  /**
   * JDBC draft specification method for allowing one thread to cancel this
   * statement which may be running in another thread. 
   * @see exjava.sql.Statement#cancel
   */
  public void cancel() {
  }
  
  /**
   * JDBC draft specification for getting the chain of warnings for this
   * statement.
   * @see exjava.sql.Statement#getWarnings
   * @return the chain of warnings
   */
  public final SQLWarning getWarnings() throws SQLException {
    return null;
  }

  /**
   * JDBC draft specification for clearing the warning chain.
   * @see exjava.sql.Statement#clearWarnings
   */
  public void clearWarnings() throws SQLException {
  }

  /**
   * JDBC 0.65 specification for setting cursor names.
   * mysql does not support cursors
   */
  public void setCursorName(String unused) throws SQLException {
    throw new SQLException("mysql does not support cursors.");
  }
}
